#!/usr/bin/env python

import sys, os
import gtk
import pango
import math
import rough

class lathe:
  def __init__(self):
    self.builder = gtk.Builder()
    self.builder.add_from_file("lathe.glade")
    self.builder.connect_signals(self)
    
    # setup the statusbar
    self.statusbar = self.builder.get_object("statusbar1")
    self.context_id = self.statusbar.get_context_id("status")
    self.statusbar_message_id = 0
    
    # setup the notebook
    self.notebook = self.builder.get_object('notebook1')

    # setup the profile tab
    self.pathstore = gtk.ListStore(str)
    self.pathview = self.builder.get_object('pathview')
    self.pathview.set_model(self.pathstore)

    self.path_column = gtk.TreeViewColumn("Profile Path")
    self.path_column.set_expand(True)
    self.path_column.set_alignment(0.5)
    self.pathview.append_column(self.path_column)


    self.path_cell = gtk.CellRendererText()
    self.path_cell.set_property("editable", True)
    self.path_cell.connect("edited", self.text_edited, self.pathstore, 0)
    self.path_cell.set_property("alignment",pango.ALIGN_CENTER)
    self.path_column.pack_start(self.path_cell, False)
    self.path_column.add_attribute(self.path_cell, "text", 0)

    self.path_entry = self.builder.get_object('path_entry')

    self.locations = []
    self.path_list = []

    self.preamble = 'G8 G20 G18 G40 G49 G64 P0.005 G80 G90 G92.1 G94'

    # setup the Settings tab
    self.feed_rate = self.builder.get_object('feed_rate')
    self.rough_doc = self.builder.get_object('rough_doc')
    self.finish_doc = self.builder.get_object('finish_doc')
    self.tool_number = self.builder.get_object('tool_number')
    self.units = 'INCH'

    # setup the drawing area
    self.arc_draw = self.builder.get_object('arc_draw')
    self.arc_draw.connect("expose-event", self.expose)
    self.start_angle = 0
    self.end_angle = 0
    
    # setup the Arc Generator page
    self.z_arc_center = self.builder.get_object('z_arc_center')
    self.arc_gcode = self.builder.get_object('arc_gcode')
    self.arc_start_point = self.builder.get_object('arc_start_point')
    self.ArcDirection = 'CCW'
    self.start_type = 'G0'

    # setup the main window
    self.window = self.builder.get_object("window1")
    self.window.set_title("Lathe Profile Generator")
    self.window.show_all()


  def addto_list(self, obj):
    if gtk.Buildable.get_name(obj) == 'path_entry':
      self.gcode = self.gcode_format(self.path_entry.get_text().upper())
    if gtk.Buildable.get_name(obj) == 'send_arc_start':
      self.gcode = self.arc_start_point.get_text()
    if gtk.Buildable.get_name(obj) == 'send_arc':
      self.gcode = self.arc_gcode.get_text()
    self.iter = self.pathstore.append([self.gcode])
    self.path_entry.set_text('')
    self.rebuild_path_list()
    self.path_entry.grab_focus()
    self.notebook.set_current_page(0)
    '''
    if self.statusbar_message_id > 0:
      self.statusbar.remove_message(self.context_id, self.statusbar_message_id)
    '''
    self.statusbar.pop(self.context_id)
  
  def gcode_format(self, obj):
    self.gcode = ''
    for i in obj:
      if i.isalpha() and obj.index(i) > 0:
        self.gcode += ' ' + i
      elif i == ' ':
        pass
      else:
        self.gcode += i
    return self.gcode

  def text_edited(self, widget, path, text, model, column):
      model[path][column] = text
      self.rebuild_path_list()

  def rebuild_path_list(self):
    self.path_list = []
    for i in range(len(self.pathstore)):
      self.iter = self.pathstore.get_iter(i)
      self.path_list.append(self.pathstore[self.iter][0])

  def on_delete_row_clicked(self, obj, data=None):
    selection = self.pathview.get_selection()
    result = selection.get_selected()
    if result[1] != None: #result could be None
      model, iter = result
      model.remove(iter)


  def on_send_to_file_btn_clicked(self, obj, data=None):
    with open('temp.ngc', 'w') as self.f:
      self.f.write(self.preamble + '\n')
      self.f.write('F' + self.feed_rate.get_text() + '\n')
      if len(self.tool_number.get_text()) > 0:
        self.f.write('T%s M6 G43\n' % self.tool_number.get_text())
      for item in self.path_list:
        self.f.write('%s\n' % item)
      self.f.write('M2')
    self.statusbar.push(self.context_id, 'File Saved')

  def on_send_to_axis_btn_clicked(self, obj, data=None):
    os.system('axis-remote temp.ngc')

  def expose(self, widget, event):
    cr = widget.window.cairo_create()

    cr.set_line_width(1)
    cr.set_source_rgb(0.0, 0.0, 0.0)
    cr.save()
            
    w = widget.allocation.width
    h = widget.allocation.height

    cr.translate(w/2, h/2)
    cr.move_to(-w/3,0)
    cr.line_to(w/3,0)
    cr.restore()
    cr.stroke()

    cr.translate(w/2, h/2)
    cr.move_to(0,-w/3)
    cr.line_to(0,w/3)
    #cr.restore()
    cr.stroke()

    cr.select_font_face('Sans')
    cr.set_font_size(12) # em-square height is 90 pixels
    cr.set_source_rgb(0.0, 0.0, 0.0) # black
    cr.move_to(-8, -w/3 - 20) # move to point (x, y)
    cr.show_text('X-')
    cr.move_to(-12, -w/3 - 5) # move to point (x, y)
    cr.show_text('180')
    cr.move_to(-3, w/3 + 12) # move to point (x, y)
    cr.show_text('0')
    cr.move_to(-6, w/3 + 24) # move to point (x, y)
    cr.show_text('X+')
    cr.move_to(w/3 + 2, 10) # move to point (x, y)
    cr.show_text('90')
    cr.move_to(w/3 + 2, -8) # move to point (x, y)
    cr.show_text('Z+')
    cr.move_to(-w/3 - 26, -8) # move to point (x, y)
    cr.show_text('Z-')
    cr.move_to(-w/3 - 26, 10) # move to point (x, y)
    cr.show_text('270')
    # move to the start point of the arc!
    #cr.move_to(0,0)
    cr.new_sub_path()
    cr.rotate(math.radians(90))
    cr.arc_negative(0, 0, w/4, math.radians(self.start_angle), math.radians(self.end_angle))
    cr.stroke_preserve()

  def on_generate_arc_clicked(self, obj, data=None):
      # generate the G code
      self.XArcCenter = float(self.builder.get_object('x_arc_center').get_text())
      self.ZArcCenter = float(self.builder.get_object('z_arc_center').get_text())
      self.ArcDiameter = float(self.builder.get_object('arc_diameter').get_text())
      self.ArcRadius = self.ArcDiameter/2
      self.ArcStart = float(self.builder.get_object('arc_start_angle').get_text())
      self.ArcEnd = float(self.builder.get_object('arc_end_angle').get_text())
      self.ArcStartPoint = self.builder.get_object('arc_start_point')
      self.ArcCode = self.builder.get_object('arc_gcode')
      
      # find the X and Z start point and offset
      if self.ArcStart <= 90: # Quadrant 1
          self.XStart = self.XArcCenter + (self.ArcRadius * math.cos(math.radians(self.ArcStart)))
          self.ZStart = self.ZArcCenter + (self.ArcRadius * math.sin(math.radians(self.ArcStart)))
          self.ArcStartPoint.set_text('%s X%3.4f Z%3.4f' %(self.start_type, self.XStart, self.ZStart))
          self.IOffset = -(self.XStart - self.XArcCenter)
          self.KOffset = -(self.ZStart - self.ZArcCenter)

      elif self.ArcStart > 90 and self.ArcStart <= 180: # Quadrant 2
          self.XStart = self.XArcCenter - (self.ArcRadius*math.sin(math.radians(self.ArcStart-90)))
          self.ZStart = self.ZArcCenter + (self.ArcRadius*math.cos(math.radians(self.ArcStart-90)))
          self.ArcStartPoint.set_text('%s X%3.4f Z%3.4f' %(self.start_type, self.XStart, self.ZStart))
          self.IOffset = abs(self.XStart - self.XArcCenter)
          self.KOffset = -(self.ZStart - self.ZArcCenter)

      elif self.ArcStart > 180 and self.ArcStart <= 270: # Quadrant 3
          self.XStart = self.XArcCenter - (self.ArcRadius*math.cos(math.radians(self.ArcStart-180)))
          self.ZStart = self.ZArcCenter - (self.ArcRadius*math.sin(math.radians(self.ArcStart-180)))
          self.ArcStartPoint.set_text('%s X%3.4f Z%3.4f' %(self.start_type, self.XStart, self.ZStart))
          self.IOffset = abs(self.XStart - self.XArcCenter)
          self.KOffset = abs(self.ZStart - self.ZArcCenter)

      elif self.ArcStart > 270 and self.ArcStart <= 360: # Quadrant 4
          self.XStart = self.XArcCenter + (self.ArcRadius*math.sin(math.radians(self.ArcStart-270)))
          self.ZStart = self.ZArcCenter - (self.ArcRadius*math.cos(math.radians(self.ArcStart-270)))
          self.ArcStartPoint.set_text('%s X%3.4f Z%3.4f' %(self.start_type, self.XStart, self.ZStart))
          self.IOffset = -(self.XStart - self.XArcCenter)
          self.KOffset = -(self.ZStart - self.ZArcCenter)

      
      # find the X and Y end point
      if self.ArcEnd <= 90: # Quadrant 1
          self.XEnd = self.XArcCenter + (self.ArcRadius * math.cos(math.radians(self.ArcEnd)))
          self.ZEnd = self.ZArcCenter + (self.ArcRadius * math.sin(math.radians(self.ArcEnd)))
          self.ArcEndPoint = 'X%3.4f Z%3.4f' %(self.XStart, self.ZStart)
      elif self.ArcEnd > 90 and self.ArcEnd <= 180: # Quadrant 2
          self.XEnd = self.XArcCenter - (self.ArcRadius*math.sin(math.radians(self.ArcEnd-90)))
          self.ZEnd = self.ZArcCenter + (self.ArcRadius*math.cos(math.radians(self.ArcEnd-90)))
          self.ArcEndPoint = 'X%3.4f Z%3.4f' %(self.XStart, self.ZStart)
      elif self.ArcEnd > 180 and self.ArcEnd <= 270: # Quadrant 3
          self.XEnd = self.XArcCenter - (self.ArcRadius*math.cos(math.radians(self.ArcEnd-180)))
          self.ZEnd = self.ZArcCenter - (self.ArcRadius*math.sin(math.radians(self.ArcEnd-180)))
          self.ArcEndPoint = 'X%3.4f Z%3.4f' %(self.XStart, self.ZStart)
      elif self.ArcEnd > 270 and self.ArcEnd <= 360: # Quadrant 4
          self.XEnd = self.XArcCenter + (self.ArcRadius*math.sin(math.radians(self.ArcEnd-270)))
          self.ZEnd = self.ZArcCenter - (self.ArcRadius*math.cos(math.radians(self.ArcEnd-270)))
          self.ArcEndPoint = 'X%3.4f Z%3.4f' %(self.XStart, self.ZStart)

      if self.ArcDirection == 'CCW': # CCW
          self.ArcCode.set_text('G2 X%.4f Z%.4f I%.4f K%.4f' \
          %(self.XEnd, self.ZEnd, self.IOffset, self.KOffset))
      elif self.ArcDirection == 'CW': # CW
          self.ArcCode.set_text('G3 X%.4f Z%.4f I%.4f K%.4f' \
          %(self.XEnd, self.ZEnd, self.IOffset, self.KOffset))

      #self.gcode = 'F' + self.FeedRateVar.get() + '\n'
      #self.gcode += 'G0 '+ self.StartPointVar.get() + '\n'
      #self.gcode += self.ArcCodeVar.get() + '\n'
      #self.gcode += 'M2'
      
  def on_show_arc_clicked(self, obj, data=None):
    self.start_angle = 360 - float(self.builder.get_object('arc_start_angle').get_text())
    self.end_angle = 360 - float(self.builder.get_object('arc_end_angle').get_text())
    self.arc_draw.queue_draw()

  def on_units_rb_toggled(self, obj):
    if obj.get_active():
      if gtk.Buildable.get_name(obj) == 'inch_rb':
        self.preamble = 'G8 G20 G18 G40 G49 G64 P0.005 G80 G90 G92.1 G94'
      if gtk.Buildable.get_name(obj) == 'metric_rb':
        self.preamble = 'G8 G21 G18 G40 G49 G64 P0.005 G80 G90 G92.1 G94'

  def on_direction_toggled(self, obj):
    if obj.get_active():
      if gtk.Buildable.get_name(obj) == 'ccw_rb':self.ArcDirection = 'CCW'
      if gtk.Buildable.get_name(obj) == 'cw_rb':self.ArcDirection = 'CW'

  def on_start_toggled(self, obj):
    if obj.get_active():
      if gtk.Buildable.get_name(obj) == 'start_g0':self.start_type = 'G0'
      if gtk.Buildable.get_name(obj) == 'start_g1':self.start_type = 'G1'

  def on_x_arc_center_activate(self, obj):
    self.z_arc_center.grab_focus()

  def on_window1_destroy(self, object, data=None):
    gtk.main_quit()

  def main(self):
    # All PyGTK applications must have a gtk.main(). Control ends here
    # and waits for an event to occur (like a key press or mouse event).
    gtk.main()


main = lathe()
gtk.main()
